package com.prolog.eis.service.out;


import com.prolog.eis.dto.strategy.ContainerInfoDto;
import com.prolog.eis.model.container.ContainerInfo;
import com.prolog.framework.core.restriction.Criteria;

import java.util.List;
import java.util.Map;

/**
 * @author zmr
 * @description: 容器信息表-服务接口类
 * @date 2023-08-30 12:24
 */
public interface ContainerInfoService {

    /**
     * 新增数据
     *
     * @param containerInfo 容器信息表
     */
    long save(ContainerInfo containerInfo);

    /**
     * 新增批量数据
     *
     * @param containerInfoList 容器信息表
     */
    long saveBatch(List<ContainerInfo> containerInfoList);

    /**
     * 查询多条数据
     *
     * @param paramMap 查询起始位置
     * @return 对象列表
     */
    List<ContainerInfo> findByMap(Map<String, Object> paramMap);

    /**
     * 查询单条数据
     *
     * @param paramMap 查询起始位置
     * @return 对象
     */
    ContainerInfo findOneByMap(Map<String, Object> paramMap);

    /**
     * 查询多条数据
     *
     * @param criteria 查询起始位置
     * @return 对象列表
     */
    List<ContainerInfo> findByCriteria(Criteria criteria);

    /**
     * 通过ID查询单条数据
     *
     * @param id 主键
     * @return 实例对象
     */
    ContainerInfo findById(int id);

    /**
     * 修改数据
     *
     * @param containerInfo 实例对象
     */
    long update(ContainerInfo containerInfo);

    /**
     * 通过主键删除数据
     *
     * @param id 主键
     */
    long deleteById(int id);

    /**
     * 通过参数删除数据
     *
     * @param paramMap 参数
     */
    long deleteByMap(Map<String, Object> paramMap);

    /**
     * 通过参数查询总条数
     *
     * @param paramMap 参数
     */
    long findCountByMap(Map<String, Object> paramMap);


    List<ContainerInfoDto> getContainerInfoDto(String warehouseArea);

    void updateByMap(Map<String, Object> paramMap, Criteria criteria);

    List<ContainerInfoDto> findAGVLContainerInfo();

    List<ContainerInfoDto> getContainerInfo();
}
