package com.prolog.eis.service.out;


import com.prolog.eis.dto.page.MergeContainerReqDto;
import com.prolog.eis.dto.page.MergeContainerRespDto;
import com.prolog.eis.model.container.ContainerTask;
import com.prolog.framework.core.restriction.Criteria;

import java.util.List;
import java.util.Map;

/**
 * @author zmr
 * @description: 出库容器任务表-服务接口类
 * @date 2023-08-30 12:24
 */
public interface ContainerTaskService {

    /**
     * 新增数据
     *
     * @param containerTask 出库容器任务表
     */
    long save(ContainerTask containerTask);

    /**
     * 新增批量数据
     *
     * @param containerTaskList 出库容器任务表
     */
    long saveBatch(List<ContainerTask> containerTaskList);

    /**
     * 查询多条数据
     *
     * @param paramMap 查询起始位置
     * @return 对象列表
     */
    List<ContainerTask> findByMap(Map<String, Object> paramMap);

    /**
     * 查询单条数据
     *
     * @param paramMap 查询起始位置
     * @return 对象
     */
    ContainerTask findOneByMap(Map<String, Object> paramMap);

    /**
     * 查询多条数据
     *
     * @param criteria 查询起始位置
     * @return 对象列表
     */
    List<ContainerTask> findByCriteria(Criteria criteria);

    /**
     * 通过ID查询单条数据
     *
     * @param id 主键
     * @return 实例对象
     */
    ContainerTask findById(int id);

    /**
     * 修改数据
     *
     * @param containerTask 实例对象
     */
    long update(ContainerTask containerTask);

    /**
     * 通过主键删除数据
     *
     * @param id 主键
     */
    long deleteById(int id);

    /**
     * 通过参数删除数据
     *
     * @param paramMap 参数
     */
    long deleteByMap(Map<String, Object> paramMap);

    /**
     * 通过参数查询总条数
     *
     * @param paramMap 参数
     */
    long findCountByMap(Map<String, Object> paramMap);

    void updateMapByCriteria(Map<String, Object> map, Criteria criteria);

    Long findCountByCriteria(Criteria criteria);

    void updateMapById(Integer id, Map<String, Object> paramMap);

    void updateStatusBatch(List<ContainerTask> tempTask, int startStatus);

    List<MergeContainerRespDto> mergeContainerList(MergeContainerReqDto req);

    void containerTaskToHis(ContainerTask containerTask);
}
