package com.prolog.eis.service.out.impl;

import com.prolog.eis.dao.out.LocationCfgMapper;
import com.prolog.eis.model.container.LocationCfg;
import com.prolog.eis.service.out.LocationCfgService;
import com.prolog.framework.core.restriction.Criteria;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;
import java.util.Map;

/**
 * @author zmr
 * @description: 关键点位配置表-服务实现类
 * @date 2023-08-30 12:24
 */
@Service
public class LocationCfgServiceImpl implements LocationCfgService {

    @Autowired
    private LocationCfgMapper locationCfgMapper;

    /**
     * 新增数据
     *
     * @param locationCfg 关键点位配置表
     */
    @Override
    public long save(LocationCfg locationCfg) {
        return locationCfgMapper.save(locationCfg);
    }

    /**
     * 新增批量数据
     *
     * @param locationCfgList 关键点位配置表
     */
    @Override
    public long saveBatch(List<LocationCfg> locationCfgList) {
        return locationCfgMapper.saveBatch(locationCfgList);
    }

    /**
     * 查询多条数据
     *
     * @param paramMap 查询起始位置
     * @return 对象列表
     */
    @Override
    public List<LocationCfg> findByMap(Map<String, Object> paramMap) {
        return locationCfgMapper.findByMap(paramMap, LocationCfg.class);
    }

    /**
     * 查询单条数据
     *
     * @param paramMap 查询起始位置
     * @return 对象
     */
    @Override
    public LocationCfg findOneByMap(Map<String, Object> paramMap) {
        List<LocationCfg> dataList = findByMap(paramMap);
        if(dataList.isEmpty()) {
            return null;
        }
        return dataList.get(0);
    }

    /**
     * 查询多条数据
     *
     * @param criteria 查询起始位置
     * @return 对象列表
     */
    @Override
    public List<LocationCfg> findByCriteria(Criteria criteria) {
        return locationCfgMapper.findByCriteria(criteria);
    }

    /**
     * 通过ID查询单条数据
     *
     * @param id 主键
     * @return 实例对象
     */
    @Override
    public LocationCfg findById(int id) {
        return locationCfgMapper.findById(id, LocationCfg.class);
    }

    /**
     * 修改数据
     *
     * @param locationCfg 实例对象
     */
    @Override
    public long update(LocationCfg locationCfg) {
        return locationCfgMapper.update(locationCfg);
    }

    /**
     * 通过主键删除数据
     *
     * @param id 主键
     */
    @Override
    public long deleteById(int id) {
        return locationCfgMapper.deleteById(id, LocationCfg.class);
    }

    /**
     * 通过参数删除数据
     *
     * @param paramMap 参数
     */
    @Override
    public long deleteByMap(Map<String, Object> paramMap) {
        return locationCfgMapper.deleteByMap(paramMap, LocationCfg.class);
    }

    /**
     * 通过参数查询总条数
     *
     * @param paramMap 参数
     */
    @Override
    public long findCountByMap(Map<String, Object> paramMap) {
        return locationCfgMapper.findCountByMap(paramMap, LocationCfg.class);
    }
}
