package com.prolog.eis.service.out.impl;

import com.prolog.eis.dao.out.StationCfgMapper;
import com.prolog.eis.model.container.StationCfg;
import com.prolog.eis.service.out.StationCfgService;
import com.prolog.framework.core.restriction.Criteria;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;
import java.util.Map;

/**
 * @author zmr
 * @description: 站台配置表-服务实现类
 * @date 2023-08-30 12:24
 */
@Service
public class StationCfgServiceImpl implements StationCfgService {

    @Autowired
    private StationCfgMapper stationCfgMapper;

    /**
     * 新增数据
     *
     * @param stationCfg 站台配置表
     */
    @Override
    public long save(StationCfg stationCfg) {
        return stationCfgMapper.save(stationCfg);
    }

    /**
     * 新增批量数据
     *
     * @param stationCfgList 站台配置表
     */
    @Override
    public long saveBatch(List<StationCfg> stationCfgList) {
        return stationCfgMapper.saveBatch(stationCfgList);
    }

    /**
     * 查询多条数据
     *
     * @param paramMap 查询起始位置
     * @return 对象列表
     */
    @Override
    public List<StationCfg> findByMap(Map<String, Object> paramMap) {
        return stationCfgMapper.findByMap(paramMap, StationCfg.class);
    }

    /**
     * 查询单条数据
     *
     * @param paramMap 查询起始位置
     * @return 对象
     */
    @Override
    public StationCfg findOneByMap(Map<String, Object> paramMap) {
        List<StationCfg> dataList = findByMap(paramMap);
        if(dataList.isEmpty()) {
            return null;
        }
        return dataList.get(0);
    }

    /**
     * 查询多条数据
     *
     * @param criteria 查询起始位置
     * @return 对象列表
     */
    @Override
    public List<StationCfg> findByCriteria(Criteria criteria) {
        return stationCfgMapper.findByCriteria(criteria);
    }

    /**
     * 通过ID查询单条数据
     *
     * @param id 主键
     * @return 实例对象
     */
    @Override
    public StationCfg findById(int id) {
        return stationCfgMapper.findById(id, StationCfg.class);
    }

    /**
     * 修改数据
     *
     * @param stationCfg 实例对象
     */
    @Override
    public long update(StationCfg stationCfg) {
        return stationCfgMapper.update(stationCfg);
    }

    /**
     * 通过主键删除数据
     *
     * @param id 主键
     */
    @Override
    public long deleteById(int id) {
        return stationCfgMapper.deleteById(id, StationCfg.class);
    }

    /**
     * 通过参数删除数据
     *
     * @param paramMap 参数
     */
    @Override
    public long deleteByMap(Map<String, Object> paramMap) {
        return stationCfgMapper.deleteByMap(paramMap, StationCfg.class);
    }

    /**
     * 通过参数查询总条数
     *
     * @param paramMap 参数
     */
    @Override
    public long findCountByMap(Map<String, Object> paramMap) {
        return stationCfgMapper.findCountByMap(paramMap, StationCfg.class);
    }
}
