package com.prolog.eis.service.shelves.impl;

import com.prolog.eis.dao.shelves.GridInfoMapper;
import com.prolog.eis.model.shelves.GridInfo;
import com.prolog.eis.service.shelves.GridInfoService;
import com.prolog.framework.core.restriction.Criteria;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;
import java.util.Map;

/**
 * @author zmr
 * @description: 格口信息表-服务实现类
 * @date 2023-08-28 11:41
 */
@Service
public class GridInfoServiceImpl implements GridInfoService {

    @Autowired
    private GridInfoMapper gridInfoMapper;

    /**
     * 新增数据
     *
     * @param gridInfo 格口信息表
     */
    @Override
    public long save(GridInfo gridInfo) {
        return gridInfoMapper.save(gridInfo);
    }

    /**
     * 新增批量数据
     *
     * @param gridInfoList 格口信息表
     */
    @Override
    public long saveBatch(List<GridInfo> gridInfoList) {
        return gridInfoMapper.saveBatch(gridInfoList);
    }

    /**
     * 查询多条数据
     *
     * @param paramMap 查询起始位置
     * @return 对象列表
     */
    @Override
    public List<GridInfo> findByMap(Map<String, Object> paramMap) {
        return gridInfoMapper.findByMap(paramMap, GridInfo.class);
    }

    /**
     * 查询单条数据
     *
     * @param paramMap 查询起始位置
     * @return 对象
     */
    @Override
    public GridInfo findOneByMap(Map<String, Object> paramMap) {
        List<GridInfo> dataList = findByMap(paramMap);
        if(dataList.isEmpty()) {
            return null;
        }
        return dataList.get(0);
    }

    /**
     * 查询多条数据
     *
     * @param criteria 查询起始位置
     * @return 对象列表
     */
    @Override
    public List<GridInfo> findByCriteria(Criteria criteria) {
        return gridInfoMapper.findByCriteria(criteria);
    }

    /**
     * 通过ID查询单条数据
     *
     * @param id 主键
     * @return 实例对象
     */
    @Override
    public GridInfo findById(int id) {
        return gridInfoMapper.findById(id, GridInfo.class);
    }

    /**
     * 修改数据
     *
     * @param gridInfo 实例对象
     */
    @Override
    public long update(GridInfo gridInfo) {
        return gridInfoMapper.update(gridInfo);
    }


    /**
     * 修改数据
     *
     */
    @Override
    public long updateMapByCriteria(Map<String,Object> params,Criteria criteria) {
        return gridInfoMapper.updateMapByCriteria(params,criteria);
    }

    /**
     * 通过主键删除数据
     *
     * @param id 主键
     */
    @Override
    public long deleteById(int id) {
        return gridInfoMapper.deleteById(id, GridInfo.class);
    }

    /**
     * 通过参数删除数据
     *
     * @param paramMap 参数
     */
    @Override
    public long deleteByMap(Map<String, Object> paramMap) {
        return gridInfoMapper.deleteByMap(paramMap, GridInfo.class);
    }

    /**
     * 通过参数查询总条数
     *
     * @param paramMap 参数
     */
    @Override
    public long findCountByMap(Map<String, Object> paramMap) {
        return gridInfoMapper.findCountByMap(paramMap, GridInfo.class);
    }
}
