package com.prolog.eis.service.wms;

import com.alibaba.fastjson.JSON;
import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.core.type.TypeReference;
import com.prolog.eis.dto.wms.WmsBindGirdDto;
import com.prolog.eis.utils.HttpUtil;
import com.prolog.framework.common.message.RestMessage;
import com.prolog.framework.utils.JsonUtils;
import com.prolog.fx.inf.core.Exception.RetryException;
import com.prolog.fx.inf.core.InterfaceType;
import com.prolog.fx.inf.core.StandardInterface;
import com.prolog.fx.inf.core.annotation.PrologInterface;
import com.prolog.fx.inf.core.dto.SimpleLog;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.web.client.RestTemplate;

import java.io.IOException;

@Service
@Slf4j
@PrologInterface(key = "WmsBindGirdBackService", name = "WMS订单绑定格口回告", type = InterfaceType.Send)
public class WmsBindGirdBackService implements StandardInterface<WmsBindGirdDto, RestMessage> {

    @Value("${wms.wms-bind-gird-back}")
    private String wmsUrl;
    @Autowired
    private RestTemplate newRestTemplate;

    @Override
    public RestMessage execute(WmsBindGirdDto dataObject) throws Exception {
        try {
            RestMessage wmsResult = newRestTemplate.postForObject(wmsUrl, dataObject, RestMessage.class);
            if (!wmsResult.isSuccess()) {
                log.error("请求wms失败" + wmsResult.getMessage());
                throw new Exception("请求wms失败" + wmsResult.getMessage());
            }
            return wmsResult;
        } catch (Exception e) {
            throw new RetryException(e.getMessage());
        }
    }

    @Override
    public SimpleLog toLog(WmsBindGirdDto dataObject) {
        SimpleLog simpleLog = new SimpleLog();
        try {
            simpleLog.setData(JsonUtils.toString(dataObject));
        } catch (JsonProcessingException e) {
            log.error(e.getMessage());
        }
        return simpleLog;
    }

    @Override
    public WmsBindGirdDto toDataObject(SimpleLog simpleLog) {
        if (simpleLog != null) {
            try {
                WmsBindGirdDto dataObject = JsonUtils.parseJsonString(simpleLog.getData(), new TypeReference<WmsBindGirdDto>() {
                });
                return dataObject;
            } catch (IOException e) {
                log.error(e.getMessage());
            }
        }
        return null;
    }
}
