package com.prolog.eis.service.wms;

import com.alibaba.fastjson.JSON;
import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.core.type.TypeReference;
import com.prolog.eis.dto.wms.WmsWarehouseChangeDto;
import com.prolog.eis.utils.HttpUtil;
import com.prolog.framework.common.message.RestMessage;
import com.prolog.framework.utils.JsonUtils;
import com.prolog.fx.inf.core.InterfaceType;
import com.prolog.fx.inf.core.StandardInterface;
import com.prolog.fx.inf.core.annotation.PrologInterface;
import com.prolog.fx.inf.core.dto.SimpleLog;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.web.client.RestTemplate;

import java.io.IOException;

@Service
@Slf4j
@PrologInterface(key = "wmsWarehouseChange", name = "WMS库存变更申请", type = InterfaceType.Send)
public class WmsWarehouseChangeService implements StandardInterface<WmsWarehouseChangeDto, RestMessage> {

    @Value("${wms.wms-warehouse-change}")
    private String eis2wmsWarehouseChange;
    @Autowired
    private RestTemplate newRestTemplate;

    @Override
    public RestMessage execute(WmsWarehouseChangeDto dataObject) throws Exception {
        String restMessage = HttpUtil.postMethod(eis2wmsWarehouseChange, JSON.toJSONString(dataObject));
        RestMessage wmsResult = JSON.parseObject(restMessage, RestMessage.class);
        if (!wmsResult.isSuccess()) {
            log.error("请求wms失败" + wmsResult.getMessage());
            throw new Exception("请求wms失败" + wmsResult.getMessage());
        }
        return wmsResult;
    }

    @Override
    public SimpleLog toLog(WmsWarehouseChangeDto dataObject) {
        SimpleLog simpleLog = new SimpleLog();
        try {
            simpleLog.setData(JsonUtils.toString(dataObject));
        } catch (JsonProcessingException e) {
            log.error(e.getMessage());
        }
//        simpleLog.setBussinessKey1(dataObject.getContainerNo());
        return simpleLog;
    }

    @Override
    public WmsWarehouseChangeDto toDataObject(SimpleLog simpleLog) {
        if (simpleLog != null) {
            try {
                WmsWarehouseChangeDto dataObject = JsonUtils.parseJsonString(simpleLog.getData(), new TypeReference<WmsWarehouseChangeDto>() {
                });
                return dataObject;
            } catch (IOException e) {
                log.error(e.getMessage());
            }
        }
        return null;
    }
}
