package com.prolog.eis.service.out;


import com.prolog.eis.dto.strategy.OrderDto;
import com.prolog.eis.model.outbound.BizOutboundTask;
import com.prolog.framework.core.restriction.Criteria;

import java.util.List;
import java.util.Map;

/**
 * @author zmr
 * @description: 出库任务表-服务接口类
 * @date 2023-08-30 12:24
 */
public interface BizOutboundTaskService {

    /**
     * 新增数据
     *
     * @param bizOutboundTask 出库任务表
     */
    long save(BizOutboundTask bizOutboundTask);

    /**
     * 新增批量数据
     *
     * @param bizOutboundTaskList 出库任务表
     */
    long saveBatch(List<BizOutboundTask> bizOutboundTaskList);

    /**
     * 查询多条数据
     *
     * @param paramMap 查询起始位置
     * @return 对象列表
     */
    List<BizOutboundTask> findByMap(Map<String, Object> paramMap);

    /**
     * 查询单条数据
     *
     * @param paramMap 查询起始位置
     * @return 对象
     */
    BizOutboundTask findOneByMap(Map<String, Object> paramMap);

    /**
     * 查询多条数据
     *
     * @param criteria 查询起始位置
     * @return 对象列表
     */
    List<BizOutboundTask> findByCriteria(Criteria criteria);


    /**
     * 查询多条数据
     *
     * @param criteria 查询起始位置
     * @return 对象列表
     */
    long findCountByCriteria(Criteria criteria);

    /**
     * 通过ID查询单条数据
     *
     * @param id 主键
     * @return 实例对象
     */
    BizOutboundTask findById(int id);

    /**
     * 修改数据
     *
     * @param bizOutboundTask 实例对象
     */
    long update(BizOutboundTask bizOutboundTask);


    long updateMapByCriteria(Map<String,Object> params,Criteria criteria);

    /**
     * 通过主键删除数据
     *
     * @param id 主键
     */
    long deleteById(int id);

    /**
     * 通过参数删除数据
     *
     * @param paramMap 参数
     */
    long deleteByMap(Map<String, Object> paramMap);

    /**
     * 通过参数查询总条数
     *
     * @param paramMap 参数
     */
    long findCountByMap(Map<String, Object> paramMap);

    List<OrderDto> getOutboundTaskByBillType(String billType, String warehouseArea, String assemblyOrderSwitch,int checkStore,int isBindGrid);

    List<OrderDto> getOutboundTaskList(String billType, String warehouseArea);

    void batchUpdate(List<BizOutboundTask> updateBizOutboundTaskList);

    List<BizOutboundTask> getAGVLPdYhOrder();

    List<OrderDto> getCreateOutboundTaskList(String billType, String warehouseArea);

    void updateById(Integer orderId, Map<String, Object> params);

    List<OrderDto> getOutboundTaskByArea(String agvl);

    List<BizOutboundTask> findOutTaskList(String warehouseArea,String billType);

    List<OrderDto> getCreateOutboundTaskLSList(String billType, String warehouseArea);

    List<OrderDto> getOutboundTaskLSList(String billType, String warehouseArea);

    void batchUpdateStatus(List<Integer> ids, Integer status);
}
