package com.prolog.eis.service.out.impl;

import com.prolog.eis.dao.out.ContainerTaskMapper;
import com.prolog.eis.dto.page.MergeContainerReqDto;
import com.prolog.eis.dto.page.MergeContainerRespDto;
import com.prolog.eis.model.container.ContainerTask;
import com.prolog.eis.service.out.ContainerTaskDismantleService;
import com.prolog.eis.service.out.ContainerTaskService;
import com.prolog.framework.core.restriction.Criteria;
import com.prolog.framework.dao.util.PageUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;
import java.util.Map;

/**
 * @author zmr
 * @description: 出库容器任务表-服务实现类
 * @date 2023-08-30 12:24
 */
@Service
public class ContainerTaskServiceImpl implements ContainerTaskService {

    @Autowired
    private ContainerTaskMapper containerTaskMapper;
    @Autowired
    private ContainerTaskDismantleService containerTaskDismantleService;

    /**
     * 新增数据
     *
     * @param containerTask 出库容器任务表
     */
    @Override
    public long save(ContainerTask containerTask) {
        return containerTaskMapper.save(containerTask);
    }

    /**
     * 新增批量数据
     *
     * @param containerTaskList 出库容器任务表
     */
    @Override
    public long saveBatch(List<ContainerTask> containerTaskList) {
        return containerTaskMapper.saveBatch(containerTaskList);
    }

    /**
     * 查询多条数据
     *
     * @param paramMap 查询起始位置
     * @return 对象列表
     */
    @Override
    public List<ContainerTask> findByMap(Map<String, Object> paramMap) {
        return containerTaskMapper.findByMap(paramMap, ContainerTask.class);
    }

    /**
     * 查询单条数据
     *
     * @param paramMap 查询起始位置
     * @return 对象
     */
    @Override
    public ContainerTask findOneByMap(Map<String, Object> paramMap) {
        List<ContainerTask> dataList = findByMap(paramMap);
        if(dataList.isEmpty()) {
            return null;
        }
        return dataList.get(0);
    }

    /**
     * 查询多条数据
     *
     * @param criteria 查询起始位置
     * @return 对象列表
     */
    @Override
    public List<ContainerTask> findByCriteria(Criteria criteria) {
        return containerTaskMapper.findByCriteria(criteria);
    }

    /**
     * 通过ID查询单条数据
     *
     * @param id 主键
     * @return 实例对象
     */
    @Override
    public ContainerTask findById(int id) {
        return containerTaskMapper.findById(id, ContainerTask.class);
    }

    /**
     * 修改数据
     *
     * @param containerTask 实例对象
     */
    @Override
    public long update(ContainerTask containerTask) {
        return containerTaskMapper.update(containerTask);
    }

    /**
     * 通过主键删除数据
     *
     * @param id 主键
     */
    @Override
    public long deleteById(int id) {
        return containerTaskMapper.deleteById(id, ContainerTask.class);
    }

    /**
     * 通过参数删除数据
     *
     * @param paramMap 参数
     */
    @Override
    public long deleteByMap(Map<String, Object> paramMap) {
        return containerTaskMapper.deleteByMap(paramMap, ContainerTask.class);
    }

    /**
     * 通过参数查询总条数
     *
     * @param paramMap 参数
     */
    @Override
    public long findCountByMap(Map<String, Object> paramMap) {
        return containerTaskMapper.findCountByMap(paramMap, ContainerTask.class);
    }

    @Override
    public void updateMapByCriteria(Map<String, Object> map, Criteria criteria) {
        containerTaskMapper.updateMapByCriteria(map, criteria);
    }

    @Override
    public Long findCountByCriteria(Criteria criteria) {
        return containerTaskMapper.findCountByCriteria(criteria);
    }

    @Override
    public void updateMapById(Integer id, Map<String, Object> paramMap) {
        containerTaskMapper.updateMapById(id, paramMap, ContainerTask.class);
    }

    @Override
    public void updateStatusBatch(List<ContainerTask> tempTask, int startStatus) {
        containerTaskMapper.updateStatusBatch(tempTask, startStatus);
    }

    @Override
    public List<MergeContainerRespDto> mergeContainerList(MergeContainerReqDto req) {
        PageUtils.startPage(req.getPageNum(),req.getPageSize());
        return containerTaskMapper.mergeContainerList(req);
    }
}
