package com.prolog.eis.service.out.impl;

import com.prolog.eis.dao.out.PrintTaskMapper;
import com.prolog.eis.model.outbound.PrintTask;
import com.prolog.eis.service.out.PrintTaskService;
import com.prolog.framework.core.restriction.Criteria;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;
import java.util.Map;

/**
 * @author zmr
 * @description: 打印任务表-服务实现类
 * @date 2023-08-30 12:24
 */
@Service
public class PrintTaskServiceImpl implements PrintTaskService {

    @Autowired
    private PrintTaskMapper printTaskMapper;

    /**
     * 新增数据
     *
     * @param printTask 打印任务表
     */
    @Override
    public long save(PrintTask printTask) {
        return printTaskMapper.save(printTask);
    }

    /**
     * 新增批量数据
     *
     * @param printTaskList 打印任务表
     */
    @Override
    public long saveBatch(List<PrintTask> printTaskList) {
        return printTaskMapper.saveBatch(printTaskList);
    }

    /**
     * 查询多条数据
     *
     * @param paramMap 查询起始位置
     * @return 对象列表
     */
    @Override
    public List<PrintTask> findByMap(Map<String, Object> paramMap) {
        return printTaskMapper.findByMap(paramMap, PrintTask.class);
    }

    /**
     * 查询单条数据
     *
     * @param paramMap 查询起始位置
     * @return 对象
     */
    @Override
    public PrintTask findOneByMap(Map<String, Object> paramMap) {
        List<PrintTask> dataList = findByMap(paramMap);
        if(dataList.isEmpty()) {
            return null;
        }
        return dataList.get(0);
    }

    /**
     * 查询多条数据
     *
     * @param criteria 查询起始位置
     * @return 对象列表
     */
    @Override
    public List<PrintTask> findByCriteria(Criteria criteria) {
        return printTaskMapper.findByCriteria(criteria);
    }

    /**
     * 通过ID查询单条数据
     *
     * @param id 主键
     * @return 实例对象
     */
    @Override
    public PrintTask findById(int id) {
        return printTaskMapper.findById(id, PrintTask.class);
    }

    /**
     * 修改数据
     *
     * @param printTask 实例对象
     */
    @Override
    public long update(PrintTask printTask) {
        return printTaskMapper.update(printTask);
    }

    /**
     * 通过主键删除数据
     *
     * @param id 主键
     */
    @Override
    public long deleteById(int id) {
        return printTaskMapper.deleteById(id, PrintTask.class);
    }

    /**
     * 通过参数删除数据
     *
     * @param paramMap 参数
     */
    @Override
    public long deleteByMap(Map<String, Object> paramMap) {
        return printTaskMapper.deleteByMap(paramMap, PrintTask.class);
    }

    /**
     * 通过参数查询总条数
     *
     * @param paramMap 参数
     */
    @Override
    public long findCountByMap(Map<String, Object> paramMap) {
        return printTaskMapper.findCountByMap(paramMap, PrintTask.class);
    }
}
