package com.prolog.eis.service.carry;


import com.prolog.eis.core.dto.route.CarryTaskCallbackDto;
import com.prolog.eis.dto.inbound.WmsCarryTaskDto;
import com.prolog.eis.model.carry.WmsCarryTask;
import com.prolog.framework.core.restriction.Criteria;

import java.util.List;
import java.util.Map;

/**
 * @author zmr
 * @description: wms下发搬运任务-服务接口类
 * @date 2023-04-23 17:30
 */
public interface WmsCarryTaskService {

    /**
     * 新增数据
     *
     * @param wmsCarryTask wms下发搬运任务
     */
    long save(WmsCarryTask wmsCarryTask);

    /**
     * 新增批量数据
     *
     * @param wmsCarryTaskList wms下发搬运任务
     */
    long saveBatch(List<WmsCarryTask> wmsCarryTaskList);

    /**
     * 查询多条数据
     *
     * @param paramMap 查询起始位置
     * @return 对象列表
     */
    List<WmsCarryTask> findByMap(Map<String, Object> paramMap);

    /**
     * 查询单条数据
     *
     * @param paramMap 查询起始位置
     * @return 对象
     */
    WmsCarryTask findOneByMap(Map<String, Object> paramMap);

    /**
     * 查询多条数据
     *
     * @param criteria 查询起始位置
     * @return 对象列表
     */
    List<WmsCarryTask> findByCriteria(Criteria criteria);

    /**
     * 通过ID查询单条数据
     *
     * @param id 主键
     * @return 实例对象
     */
    WmsCarryTask findById(int id);

    /**
     * 修改数据
     *
     * @param wmsCarryTask 实例对象
     */
    long update(WmsCarryTask wmsCarryTask);

    /**
     * 通过主键删除数据
     *
     * @param id 主键
     */
    long deleteById(int id);

    /**
     * 通过参数删除数据
     *
     * @param paramMap 参数
     */
    long deleteByMap(Map<String, Object> paramMap);

    /**
     * 通过参数查询总条数
     *
     * @param paramMap 参数
     */
    long findCountByMap(Map<String, Object> paramMap);

    void doJdTaskCallBack(CarryTaskCallbackDto carryTaskCallbackDto);

    void wmsTaskCbWms() throws Exception;

    void wmsCarryTaskCallBack(WmsCarryTask carryTask) throws Exception;

    void wmsCarryTask() throws Exception;

    void saveCarryTask(List<WmsCarryTaskDto> req);
}
