package com.prolog.eis.service.outbound;


import com.prolog.eis.core.dto.rcs.RcsSendBindReqDto;
import com.prolog.eis.core.model.biz.route.ContainerLocation;
import com.prolog.eis.dto.outbound.*;
import com.prolog.eis.dto.pms.PmsRobotTaskFinishReq;
import com.prolog.eis.dto.strategy.OrderDto;
import com.prolog.eis.dto.wms.ItemContainerDto;
import com.prolog.eis.model.container.ContainerTask;
import com.prolog.eis.model.outbound.BizOutboundTask;
import com.prolog.eis.outbound_dispatch.DispatchStrategy;
import com.prolog.framework.common.message.RestMessage;
import com.prolog.framework.core.restriction.Criteria;

import java.util.List;
import java.util.Map;

/**
 * @author zmr
 * @description: 出库任务表-服务接口类
 * @date 2023-07-06 11:36
 */
public interface BizOutboundTaskService {

    /**
     * 新增数据
     *
     * @param bizOutboundTask 出库任务表
     */
    long save(BizOutboundTask bizOutboundTask);

    /**
     * 新增批量数据
     *
     * @param bizOutboundTaskList 出库任务表
     */
    long saveBatch(List<BizOutboundTask> bizOutboundTaskList);

    /**
     * 查询多条数据
     *
     * @param paramMap 查询起始位置
     * @return 对象列表
     */
    List<BizOutboundTask> findByMap(Map<String, Object> paramMap);

    /**
     * 查询单条数据
     *
     * @param paramMap 查询起始位置
     * @return 对象
     */
    BizOutboundTask findOneByMap(Map<String, Object> paramMap);

    /**
     * 查询多条数据
     *
     * @param criteria 查询起始位置
     * @return 对象列表
     */
    List<BizOutboundTask> findByCriteria(Criteria criteria);

    /**
     * 通过ID查询单条数据
     *
     * @param id 主键
     * @return 实例对象
     */
    BizOutboundTask findById(int id);

    /**
     * 修改数据
     *
     * @param bizOutboundTask 实例对象
     */
    long update(BizOutboundTask bizOutboundTask);

    /**
     * 通过主键删除数据
     *
     * @param id 主键
     */
    long deleteById(int id);

    /**
     * 通过参数删除数据
     *
     * @param paramMap 参数
     */
    long deleteByMap(Map<String, Object> paramMap);

    /**
     * 通过参数查询总条数
     *
     * @param paramMap 参数
     */
    long findCountByMap(Map<String, Object> paramMap);

    void emptyContainerOutBound(int num,String targetLocation,int flag) throws Exception;

    long emptyContainerNum();

    void specifyContainerCarry(SpecifyContainerCarryDto specifyContainerCarryDto) throws Exception;

    void wmsOutBound(List<OutBoundDto> outBoundDtos) throws Exception;

    void taskFinishConfirm(List<ConfirmDto> confirmDtos) throws Exception;

    RestMessage taskCancel(List<CancelDto> cancelList);

    RestMessage updatePriority(List<PriorityDto> priorityList);

    RestMessage inventoryAudit(List<InventoryAuditDto> inventoryAuditDtos) throws Exception;

    RestMessage inventoryLock(List<InventoryLockDto> inventoryLockDtos);

    List<OrderDto> getOutboundTaskByBillType(String billType, String warehouseArea, DispatchStrategy dispatchStrategy) throws Exception;

    void batchUpdate(List<BizOutboundTask> updateBizOutboundTaskList);

    List<ContainerLocation> freeContainer(FreeContainerDto freeContainerDto) throws Exception;

    void robotTaskFinish(PmsRobotTaskFinishReq pmsRobotTaskFinishReq) throws Exception;

    List<WmsOutCbReqDto> setWmsOutBackReqDto(List<ContainerTask> containerTaskList);

    public void outboundConfirm(List<ContainerTask> containerTaskList, PmsRobotTaskFinishReq pmsRobotTaskFinishReq) throws Exception;

    void sendEmptyToCache() throws Exception;

    void moveEmptyContainer() throws Exception;

    List<ItemContainerDto> itemContainer(ItemContainerDto req) throws Exception;

    void sendLedMsg(RcsSendBindReqDto rcsSendBindReqDto);
}
