package com.prolog.eis.service.wms;

import com.alibaba.fastjson.JSON;
import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.core.type.TypeReference;
import com.prolog.eis.dto.wms.WmsContainerStatusUpdateDto;
import com.prolog.eis.utils.HttpUtil;
import com.prolog.framework.common.message.RestMessage;
import com.prolog.framework.utils.JsonUtils;
import com.prolog.fx.inf.core.InterfaceType;
import com.prolog.fx.inf.core.StandardInterface;
import com.prolog.fx.inf.core.annotation.PrologInterface;
import com.prolog.fx.inf.core.dto.SimpleLog;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.web.client.RestTemplate;

import java.io.IOException;

@Service
@Slf4j
@PrologInterface(key = "wmsContainerStatusUpdate", name = "WMS托盘状态更新", type = InterfaceType.Send)
public class WmsContainerStatusUpdateService implements StandardInterface<WmsContainerStatusUpdateDto, RestMessage> {

    @Value("${wms.wms-container-status-update}")
    private String eis2wmsContainerStatusUpdate;
    @Autowired
    private RestTemplate newRestTemplate;

    @Override
    public RestMessage execute(WmsContainerStatusUpdateDto dataObject) throws Exception {
        RestMessage wmsResult = newRestTemplate.postForObject(eis2wmsContainerStatusUpdate, dataObject,RestMessage.class);
        if (!wmsResult.isSuccess()) {
            log.error("请求wms失败" + wmsResult.getMessage());
            throw new Exception("请求wms失败" + wmsResult.getMessage());
        }
        return wmsResult;
    }

    @Override
    public SimpleLog toLog(WmsContainerStatusUpdateDto dataObject) {
        SimpleLog simpleLog = new SimpleLog();
        try {
            simpleLog.setData(JsonUtils.toString(dataObject));
        } catch (JsonProcessingException e) {
            log.error(e.getMessage());
        }
//        simpleLog.setBussinessKey1(dataObject.getContainerNo());
        return simpleLog;
    }

    @Override
    public WmsContainerStatusUpdateDto toDataObject(SimpleLog simpleLog) {
        if (simpleLog != null) {
            try {
                WmsContainerStatusUpdateDto dataObject = JsonUtils.parseJsonString(simpleLog.getData(), new TypeReference<WmsContainerStatusUpdateDto>() {
                });
                return dataObject;
            } catch (IOException e) {
                log.error(e.getMessage());
            }
        }
        return null;
    }
}
