package com.prolog.eis.service.wms;

import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.core.type.TypeReference;
import com.prolog.eis.dto.inbound.WmsInBackReqDto;
import com.prolog.eis.dto.result.WmsResult;
import com.prolog.framework.common.message.RestMessage;
import com.prolog.framework.utils.JsonUtils;
import com.prolog.fx.inf.core.Exception.RetryException;
import com.prolog.fx.inf.core.InterfaceType;
import com.prolog.fx.inf.core.StandardInterface;
import com.prolog.fx.inf.core.annotation.PrologInterface;
import com.prolog.fx.inf.core.dto.SimpleLog;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.web.client.RestTemplate;

import java.io.IOException;
import java.util.List;

@Service
@Slf4j
@PrologInterface(key = "wmsInBack", name = "WMS入库回告", type = InterfaceType.Send)
public class WmsInBackService implements StandardInterface<WmsInBackReqDto, RestMessage> {

    @Value("${wms.wms-in-back}")
    private String eis2wmsInBack;
    @Autowired
    private RestTemplate newRestTemplate;

    @Override
    public RestMessage execute(WmsInBackReqDto dataObject) throws Exception {
        try {
            RestMessage wmsResult = newRestTemplate.postForObject(eis2wmsInBack, dataObject, RestMessage.class);
            if (!wmsResult.isSuccess()) {
                log.error("请求wms失败" + wmsResult.getMessage());
                throw new Exception("请求wms失败" + wmsResult.getMessage());
            }
            return wmsResult;
        } catch (Exception e) {
            throw new RetryException(e.getMessage());
        }
    }

    @Override
    public SimpleLog toLog(WmsInBackReqDto dataObject) {
        SimpleLog simpleLog = new SimpleLog();
        try {
            simpleLog.setData(JsonUtils.toString(dataObject));
        } catch (JsonProcessingException e) {
            log.error(e.getMessage());
        }
//        simpleLog.setBussinessKey1(dataObject.getContainerNo());
        return simpleLog;
    }

    @Override
    public WmsInBackReqDto toDataObject(SimpleLog simpleLog) {
        if (simpleLog != null) {
            try {
                WmsInBackReqDto dataObject = JsonUtils.parseJsonString(simpleLog.getData(), new TypeReference<WmsInBackReqDto>() {
                });
                return dataObject;
            } catch (IOException e) {
                log.error(e.getMessage());
            }
        }
        return null;
    }
}
